import { BIP47Factory } from '@samouraiwallet/bip47';
import * as utils from '@samouraiwallet/bip47/utils';
import { bitcoinMessageFactory } from '@samouraiwallet/bitcoinjs-message';
import { pipe, Either, Boolean } from 'effect';
import * as S from '@effect/schema/Schema';
import { ArrayFormatter } from '@effect/schema';
import { CallbackUri, GenerateURIArgs, Proof } from './decoders.js';
const networks = utils.networks;
const getNetwork = (networkString) => {
    switch (networkString) {
        case 'bitcoin':
            return Either.right(networks.bitcoin);
        case 'testnet':
            return Either.right(networks.testnet);
        case 'regtest':
            return Either.right(networks.regtest);
        default:
            return Either.left('Invalid bitcoin network');
    }
};
export class Auth47Verifier {
    constructor(ecc, callbackUri) {
        this.validateProof = (networkString) => (proof) => pipe(getNetwork(networkString), Either.flatMap((network) => pipe(proof.kind === 'NymProof'
            ? this.bip47.fromBase58(proof.nym, network).getNotificationAddress()
            : proof.address, (notifAddress) => this.bitcoinjsMessage.verify(proof.challenge, notifAddress, proof.signature, network.messagePrefix), Boolean.match({
            onFalse: () => Either.left('invalid signature'),
            onTrue: () => Either.right(proof)
        }))));
        this.bip47 = BIP47Factory(ecc);
        this.bitcoinjsMessage = bitcoinMessageFactory(ecc);
        this.callbackUri = S.decodeUnknownEither(CallbackUri)(callbackUri).pipe(Either.mapLeft(ArrayFormatter.formatErrorSync), Either.mapLeft((e) => e[0].message), Either.match({
            onLeft: (e) => { throw e; },
            onRight: (uri) => uri
        }));
    }
    generateURI(args) {
        return pipe(args, S.decodeUnknownSync(GenerateURIArgs), (decoded) => {
            const uri = new URL(`auth47://${decoded.nonce}`);
            uri.searchParams.set('c', this.callbackUri);
            if (decoded.expires)
                uri.searchParams.set('e', decoded.expires.toString(10));
            if (decoded.resource)
                uri.searchParams.set('r', decoded.resource);
            return decodeURIComponent(uri.toString());
        });
    }
    verifyProof(proof, networkString = 'bitcoin') {
        return pipe(S.decodeUnknownEither(Proof)(proof), Either.mapLeft(ArrayFormatter.formatErrorSync), Either.mapLeft((e) => e[0].message), Either.flatMap(this.validateProof(networkString)), Either.match({
            onLeft: (e) => ({ result: 'error', error: e }),
            onRight: (proof) => ({ result: 'ok', data: proof })
        }));
    }
}
//# sourceMappingURL=index.js.map